import openai


class ChatGPT:

    PROMPT_PREFIX = (
        "Below is the description of GitHub pull requests," +
        "Do not return me any English sentence." +
        "Summarize the following description into a list of points and translate the summary into simplified Chinese. "
        )

    def __init__(self, api_key: str, model: str="text-davinci-003"):
        self.api_key = api_key
        self.model = model
        self.set_credentials()

    def set_credentials(self):
        openai.api_key = self.api_key

    def summarize(self, summary, max_tokens: int=1024):
        # Set up the model and prompt
        prompt = f"{self.PROMPT_PREFIX}\n{summary}"

        # Generate a response
        completion = openai.Completion.create(
            engine=self.model,
            prompt=prompt,
            max_tokens=max_tokens,
            n=1,
            stop=None,
            temperature=0.5,
        )
        response = completion.choices[0].text
        return response



import os.path as osp
import json
import os

HOME_DIRECTORY = osp.expanduser('~')

class CredentialManager:

    CREDENTIAL_PATH = osp.join(HOME_DIRECTORY, '.cache/whid/credentials.json')

    def __init__(self):
        self.openai_secret_key = None
        self.github_access_token = None

    def load_credential(self):
        if osp.exists(self.CREDENTIAL_PATH):
            with open(self.CREDENTIAL_PATH, 'r') as f:
                data = json.load(f)
                self.openai_secret_key = data['credentials']['openai']
                self.github_access_token = data['credentials']['github']
            return True
        else:
            return False

    def prompt_user_inputs(self):
        openai_secret_key = input("Enter your OpenAI Secret Key: ")
        github_access_token = input("Enter your GitHub Access Token: ")
        self.openai_secret_key = openai_secret_key
        self.github_access_token = github_access_token

        credentials = {
            'credentials': {
                'openai': self.openai_secret_key,
                'github': self.github_access_token
            }
        }

        cache_path = osp.dirname(self.CREDENTIAL_PATH)
        if not osp.exists(cache_path):
            os.makedirs(cache_path, exist_ok=True)

        with open(self.CREDENTIAL_PATH, 'w') as f:
            json.dump(credentials, f, indent = 4) 
            print(f'Your crendentials have been saved in {self.CREDENTIAL_PATH}')







import requests
from datetime import date, timedelta



class GithubGraphQLClient:

    GITHUB_GRAPHQL_API = "https://api.github.com/graphql"

    def __init__(self, access_token):
        self._access_token = access_token
    
    def query(self, query_string):
        headers = {"Authorization": f"Bearer {self._access_token}"}
        data = {"query": query_string}
        response = requests.post(self.GITHUB_GRAPHQL_API, json=data, headers=headers).json()
        return response

    def get_user_name(self):
        query = """
        {
            viewer { 
                login
            }
        }
        """
        response = self.query(query_string=query)
        return response['data']['viewer']['login']

    def get_pull_requests(self, user_name: str, last_n: int = 10, cursor: str = None):
        if cursor is None:
            offset = ""
        else:
            offset = f"before: \"{cursor}\""

        query = f"""
        {{
            user(login: "{user_name}") {{
                pullRequests(last: {last_n} {offset}) {{
                    totalCount
                    edges {{
                        cursor
                        node {{
                            createdAt
                            title
                            body
                            baseRepository {{
                                owner {{
                                    login
                                    id
                                }}
                                name
                                id
                            }}
                        }}
                    }}
                }}
            }}
        }}
        """
        
        response = self.query(query_string=query)
        return response



import click
from whid.core import GithubGraphQLClient, ChatGPT, CredentialManager
from datetime import datetime, timedelta

@click.command()
@click.option('-d', '--days', type=int, default=1, help="The last number of days to trace, set to 1 by default. A large number might cause ChatGPT to reject the request.")
def cli(days):
    cm = CredentialManager()
    success = cm.load_credential()

    if not success:
        cm.prompt_user_inputs()

    gh_client = GithubGraphQLClient(access_token=cm.github_access_token)
    user_name = gh_client.get_user_name()
    

    pr_info_list = []

    # github api uses UTC timezone
    current_datetime = datetime.utcnow()
    end_datetime = current_datetime - timedelta(days=days)

    # some bookkeeping variables
    cursor = None
    search_flag = True

    while search_flag:
        # get the pull requests
        response = gh_client.get_pull_requests(user_name, last_n=10, cursor=cursor)

        # iterate and extract data
        for i in range(-10, 0):
            pull_request = response['data']['user']['pullRequests']['edges'][i]

            # check timestamp
            created_at = pull_request['node']['createdAt']
            created_datetime = datetime.strptime(created_at, '%Y-%m-%dT%H:%M:%SZ')

            if created_datetime < end_datetime:
                search_flag = False
                break

            # get information
            body = pull_request['node']['body']
            title = pull_request['node']['title']
            pr_info = f'{title}\n{body}'
            pr_info_list.append(pr_info)

            # update cursor for pagination
            cursor = pull_request['cursor']

    pr_summary = '\n'.join(pr_info_list)
    chatGPT = ChatGPT(api_key=cm.openai_secret_key)
    summary = chatGPT.summarize(pr_summary)

    print("===== ChatGPT Output =====")
    print(summary)

if __name__ == '__main__':
    cli()


from setuptools import setup, find_packages

def fetch_requirements(path):
    with open(path, 'r') as fd:
        return [r.strip() for r in fd.readlines()]


def fetch_readme():
    with open('README.md', encoding='utf-8') as f:
        return f.read()


def get_version():
    with open('version.txt') as f:
        return f.read().strip()

setup(
    name='gh-whid',
    version=get_version(),
    packages=find_packages(exclude=(
        'build',
        'docker',
        'tests',
        'docs',
        'examples',
        '*.egg-info',
    )),
    description='What have I done - Summarizer for Your GitHub Pull Request Activities',
    long_description=fetch_readme(),
    long_description_content_type='text/markdown',
    license='Apache Software License 2.0',
    url='https://github.com/hpcaitech/gh-whid',
    project_urls={
        'Github': 'https://github.com/hpcaitech/gh-whid',
    },
    install_requires=fetch_requirements('requirements.txt'),
    python_requires='>=3.6',
    entry_points='''
        [console_scripts]
        whid=whid.cli:cli
    ''',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Environment :: GPU :: NVIDIA CUDA',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: System :: Distributed Computing',
    ],
)